<?php
/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use FiloBlu\OrderAdditionalData\Model\AdditionalDataFactory;
use FiloBlu\OrderAdditionalData\Model\ItemDataFactory;
use FiloBlu\OrderAdditionalData\Model\ProductOptionFactory;
use Magento\AdditionalData\Api\Data\MessageInterface;
use Magento\Catalog\Api\ProductCustomOptionRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Sales\Api\Data\ItemExtension;
use Magento\Sales\Api\Data\ItemExtensionFactory;
use Magento\Sales\Api\Data\OrderItemExtensionFactory;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Sales\Model\ResourceModel\Order\Item\Collection;
use Magento\Store\Model\ScopeInterface;
use function unserialize;

/**
 * Class ItemGet
 * Load Additional Data when getting Order Items by Repository
 * @package FiloBlu\OrderAdditionalData\Model\Plugin
 */
class ItemGet
{
    /** @var ItemDataFactory */
    protected $itemDataFactory;

    /** @var ItemExtensionFactory */
    protected $itemExtensionFactory;


    /** @var AdditionalDataFactory */
    protected $_additionalDataFactory;

    /**
     * @var ProductCustomOptionRepositoryInterface
     */
    protected $_customOptionRepository;
    /**
     * @var ProductOptionFactory
     */
    protected $_productOptionFactory;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * ItemGet constructor.
     * @param Context $context
     * @param ItemDataFactory $itemDataFactory
     * @param OrderItemExtensionFactory $itemExtensionFactory
     * @param AdditionalDataFactory $additionalDataFactory
     * @param ProductOptionFactory $productOptionFactory
     * @param ProductCustomOptionRepositoryInterface $customOptionRepository
     */
    public function __construct(
        Context $context,
        ItemDataFactory $itemDataFactory,
        OrderItemExtensionFactory $itemExtensionFactory,
        AdditionalDataFactory $additionalDataFactory,
        ProductOptionFactory $productOptionFactory,
        ProductCustomOptionRepositoryInterface $customOptionRepository
    )
    {
        $this->itemDataFactory = $itemDataFactory;
        $this->itemExtensionFactory = $itemExtensionFactory;
        $this->_additionalDataFactory = $additionalDataFactory;
        $this->scopeConfig = $context->getScopeConfig();
        $this->_productOptionFactory = $productOptionFactory;
        $this->_customOptionRepository = $customOptionRepository;
    }

    /**
     * @param OrderItemRepositoryInterface $subject
     * @param Collection $collection
     * @return Collection
     */
    public function afterGetList(
        OrderItemRepositoryInterface $subject,
        Collection $collection
    )
    {
        /** @var  $item */
        foreach ($collection->getItems() as $item) {
            $this->afterGet($subject, $item);
        }
        return $collection;
    }

    /**
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $resultItem
     * @return OrderItemInterface
     */
    public function afterGet(
        OrderItemRepositoryInterface $subject,
        OrderItemInterface $resultItem
    )
    {
        return $this->getItemAdditionalData($resultItem);
    }

    /**
     * @param OrderItemInterface $item
     * @return OrderItemInterface
     */
    protected function getItemAdditionalData(OrderItemInterface $item)
    {
        $extensionAttributes = $item->getExtensionAttributes();
        if ($extensionAttributes && $extensionAttributes->getFbOad()) {
            return $item;
        }
        //$additionalData=$this->objectFactory->create();
        $additionalData = [];
        try {
            /** @var MessageInterface $AdditionalData */
            $itemDataObject = $this->itemDataFactory->create()->load($item->getItemId(), 'item_id');

            if ($itemDataObject && $itemDataObject->getId() > 0 && $itemDataObject->getAdditionalData() !== null) {
                $additionalData = $this->unserialize($itemDataObject->getAdditionalData());
            }
        } catch (NoSuchEntityException $e) {
            return $item;
        }
        /** @var ItemExtension $itemExtension */
        $itemExtension = $extensionAttributes ?: $this->itemExtensionFactory->create();
        $item_fields = $this->scopeConfig->getValue('oad/general/item_data_fields', ScopeInterface::SCOPE_STORE);
        if ($item_fields) {
            $item_fields = $this->unserialize($item_fields);
            foreach ($item_fields as $key => $item_field) {
                if ($item_field['object_type'] !== 'order_item') {
                    unset($item_fields[$key]);
                }
            }
        }
        if ($item_fields) {
            foreach ($item_fields as $item_field) {
                $additionalData[$item_field['fieldname']] = $item->getData($item_field['object_field']);
            }
        }
        $additionalDataObject = $this->_additionalDataFactory->create();
        foreach ($additionalData as $field => $value) {
            $additionalDataObject->setCustomAttribute($field, $value);
        }
        $itemExtension->setFbOad($additionalDataObject);
        $itemExtension->setData('kpi_full_price', (float)$item->getData('kpi_full_price'));

        if ($item->getProduct()) {
            $itemExtension->setData('base_sku', $item->getProduct()->getSku());
        }

        $allProductOptions = $item->getProductOptions();
        if (isset($allProductOptions['options']) && !empty($allProductOptions['options'])) {
            $productOptions = $allProductOptions['options'];
            $productOptionsArray = [];
            foreach ($productOptions as $key => $product_option) {
                if ($item->getProduct()) {
                    $productOptionsArray[$key] = $this->_productOptionFactory->create();

                    $custom_option = $this->_customOptionRepository->get(
                        $item->getProduct()->getSku(),
                        $product_option['option_id']
                    );

                    $default_value = $product_option['value'];
                    $option_sku = '';
                    $values = $custom_option->getValues();
                    if ($values) {
                        foreach ($values as $value) {
                            if ($value->getOptionTypeId() === $product_option['option_value']) {
                                $default_value = $value->getDefaultTitle();
                                $option_sku = $value->getSku();
                                break;
                            }
                        }
                    }
                    $productOptionsArray[$key]->setData('code', $custom_option->getCode());
                    $productOptionsArray[$key]->setData('value_default_title', $default_value);
                    $productOptionsArray[$key]->setData('sku', $option_sku);
                    foreach ($product_option as $field => $value) {
                        $productOptionsArray[$key]->setData($field, $value);
                    }
                }
            }
            $itemExtension->setProductOptions($productOptionsArray);
        }
        if (array_key_exists('giftcard_created_codes', $allProductOptions)) {
            $itemExtension->setData('giftcard_created_codes', $allProductOptions['giftcard_created_codes']);
        }
        $item->setExtensionAttributes($itemExtension);
        return $item;
    }

    /**
     * @param $data
     * @return mixed
     */
    public function unserialize($data)
    {
        if (interface_exists(SerializerInterface::class) || class_exists(SerializerInterface::class)) {

            return ObjectManager::getInstance()->create(SerializerInterface::class)
                ->unserialize($data);
        }

        return unserialize($data);
    }
}
